@@ -5,6 +5,7 @@ import tempfile
 from pathlib import Path
 
 import pandas as pd
+from dask import dataframe as dd
 
 from featuretools.entityset.relationship import Relationship
 from featuretools.entityset.serialize import FORMATS
@@ -125,15 +126,20 @@ def read_entity_data(description, path):
     file = os.path.join(path, description['loading_info']['location'])
     kwargs = description['loading_info'].get('params', {})
     load_format = description['loading_info']['type']
+    entity_type = description['loading_info'].get('entity_type', 'pandas')
+    if entity_type == 'dask':
+        lib = dd
+    else:
+        lib = pd
     if load_format == 'csv':
-        dataframe = pd.read_csv(
+        dataframe = lib.read_csv(
             file,
             engine=kwargs['engine'],
             compression=kwargs['compression'],
             encoding=kwargs['encoding'],
         )
     elif load_format == 'parquet':
-        dataframe = pd.read_parquet(file, engine=kwargs['engine'])
+        dataframe = lib.read_parquet(file, engine=kwargs['engine'])
     elif load_format == 'pickle':
         dataframe = pd.read_pickle(file, **kwargs)
     else:
@@ -152,7 +158,12 @@ def read_entity_data(description, path):
             return tuple(float(y) for y in x[1:-1].split(","))
 
         for column in latlongs:
-            dataframe[column] = dataframe[column].apply(parse_latlong)
+            if entity_type == 'dask':
+                meta = (column, tuple([float, float]))
+                dataframe[column] = dataframe[column].apply(parse_latlong,
+                                                            meta=meta)
+            else:
+                dataframe[column] = dataframe[column].apply(parse_latlong)
 
     return dataframe
 
