@@ -2,6 +2,7 @@ import copy
 import logging
 from collections import defaultdict
 
+import dask.dataframe as dd
 import numpy as np
 import pandas as pd
 from pandas.api.types import is_dtype_equal, is_numeric_dtype
@@ -39,8 +40,8 @@ class EntitySet(object):
 
                 entities (dict[str -> tuple(pd.DataFrame, str, str, dict[str -> Variable])]): dictionary of
                     entities. Entries take the format
-                    {entity id -> (dataframe, id column, (time_column), (variable_types))}.
-                    Note that time_column and variable_types are optional.
+                    {entity id -> (dataframe, id column, (time_index), (variable_types), (make_index))}.
+                    Note that time_index, variable_types and make_index are optional.
 
                 relationships (list[(str, str, str, str)]): List of relationships
                     between entities. List items are a tuple with the format
@@ -69,17 +70,21 @@ class EntitySet(object):
         for entity in entities:
             df = entities[entity][0]
             index_column = entities[entity][1]
-            time_column = None
+            time_index = None
             variable_types = None
+            make_index = None
             if len(entities[entity]) > 2:
-                time_column = entities[entity][2]
+                time_index = entities[entity][2]
             if len(entities[entity]) > 3:
                 variable_types = entities[entity][3]
+            if len(entities[entity]) > 4:
+                make_index = entities[entity][4]
             self.entity_from_dataframe(entity_id=entity,
                                        dataframe=df,
                                        index=index_column,
-                                       time_index=time_column,
-                                       variable_types=variable_types)
+                                       time_index=time_index,
+                                       variable_types=variable_types,
+                                       make_index=make_index)
 
         for relationship in relationships:
             parent_variable = self[relationship[0]][relationship[1]]
@@ -254,8 +259,9 @@ class EntitySet(object):
         # default to object dtypes for discrete variables, but
         # indexes/ids default to ints. In this case, we convert
         # the empty column's type to int
-        if (child_e.df.empty and child_e.df[child_v].dtype == object and
-                is_numeric_dtype(parent_e.df[parent_v])):
+        if isinstance(child_e.df, pd.DataFrame) and \
+                (child_e.df.empty and child_e.df[child_v].dtype == object and
+                 is_numeric_dtype(parent_e.df[parent_v])):
             child_e.df[child_v] = pd.Series(name=child_v, dtype=np.int64)
 
         parent_dtype = parent_e.df[parent_v].dtype
@@ -484,6 +490,12 @@ class EntitySet(object):
                 if variable_type == vtypes.DatetimeTimeIndex:
                     raise ValueError("DatetimeTimeIndex variable %s must be set using time_index parameter" % (variable))
 
+        if len(self.entities) > 0:
+            if not isinstance(dataframe, type(self.entities[0].df)):
+                raise ValueError("All entity dataframes must be of the same type. "
+                                 "Cannot add entity of type {} to an entityset with existing entities "
+                                 "of type {}".format(type(dataframe), type(self.entities[0].df)))
+
         entity = Entity(
             entity_id,
             dataframe,
@@ -630,7 +642,7 @@ class EntitySet(object):
             drop_duplicates(index, keep='first')[selected_variables]
 
         if make_time_index:
-            new_entity_df2.rename(columns={base_time_index: new_entity_time_index}, inplace=True)
+            new_entity_df2 = new_entity_df2.rename(columns={base_time_index: new_entity_time_index})
         if make_secondary_time_index:
             assert len(make_secondary_time_index) == 1, "Can only provide 1 secondary time index"
             secondary_time_index = list(make_secondary_time_index.keys())[0]
@@ -639,12 +651,11 @@ class EntitySet(object):
             secondary_df = new_entity_df. \
                 drop_duplicates(index, keep='last')[secondary_variables]
             if new_entity_secondary_time_index:
-                secondary_df.rename(columns={secondary_time_index: new_entity_secondary_time_index},
-                                    inplace=True)
+                secondary_df = secondary_df.rename(columns={secondary_time_index: new_entity_secondary_time_index})
                 secondary_time_index = new_entity_secondary_time_index
             else:
                 new_entity_secondary_time_index = secondary_time_index
-            secondary_df.set_index(index, inplace=True)
+            secondary_df = secondary_df.set_index(index)
             new_entity_df = new_entity_df2.join(secondary_df, on=index)
         else:
             new_entity_df = new_entity_df2
@@ -782,9 +793,17 @@ class EntitySet(object):
             if entity.last_time_index is None:
                 if entity.time_index is not None:
                     lti = entity.df[entity.time_index].copy()
+                    if isinstance(entity.df, dd.DataFrame):
+                        # The current Dask implementation doesn't set the index of the dataframe
+                        # to the entity's index, so we have to do it manually here
+                        lti.index = entity.df[entity.index].copy()
                 else:
                     lti = entity.df[entity.index].copy()
-                    lti[:] = None
+                    if isinstance(entity.df, dd.DataFrame):
+                        lti.index = entity.df[entity.index].copy()
+                        lti = lti.apply(lambda x: None)
+                    else:
+                        lti[:] = None
                 entity.last_time_index = lti
 
             if entity.id in children:
@@ -809,21 +828,45 @@ class EntitySet(object):
                         continue
                     link_var = child_vars[entity.id][child_e.id].id
 
-                    lti_df = pd.DataFrame({'last_time': child_e.last_time_index,
-                                           entity.index: child_e.df[link_var]})
+                    if isinstance(child_e.last_time_index, dd.Series):
+                        to_join = child_e.df[link_var]
+                        to_join.index = child_e.df[child_e.index]
 
-                    # sort by time and keep only the most recent
-                    lti_df.sort_values(['last_time', entity.index],
-                                       kind="mergesort", inplace=True)
+                        lti_df = child_e.last_time_index.to_frame(name='last_time').join(
+                            to_join.to_frame(name=entity.index)
+                        )
+                        new_index = lti_df.index.copy()
+                        new_index.name = None
+                        lti_df.index = new_index
+                        lti_df = lti_df.groupby(lti_df[entity.index]).agg('max')
 
-                    lti_df.drop_duplicates(entity.index,
-                                           keep='last',
-                                           inplace=True)
+                        lti_df = entity.last_time_index.to_frame(name='last_time_old').join(lti_df)
+
+                    else:
+                        lti_df = pd.DataFrame({'last_time': child_e.last_time_index,
+                                               entity.index: child_e.df[link_var]})
+
+                        # sort by time and keep only the most recent
+                        lti_df.sort_values(['last_time', entity.index],
+                                           kind="mergesort", inplace=True)
+
+                        lti_df.drop_duplicates(entity.index,
+                                               keep='last',
+                                               inplace=True)
+
+                        lti_df.set_index(entity.index, inplace=True)
+                        lti_df = lti_df.reindex(entity.last_time_index.index)
+                        lti_df['last_time_old'] = entity.last_time_index
+                    if not isinstance(lti_df, dd.DataFrame) and lti_df.empty:
+                        # Pandas errors out if it tries to do fillna and then max on an empty dataframe
+                        lti_df = pd.Series()
+                    else:
+                        lti_df['last_time'] = lti_df['last_time'].astype('datetime64[ns]')
+                        lti_df['last_time_old'] = lti_df['last_time_old'].astype('datetime64[ns]')
+                        lti_df = lti_df.fillna(pd.to_datetime('1800-01-01 00:00')).max(axis=1)
+                        lti_df = lti_df.replace(pd.to_datetime('1800-01-01 00:00'), pd.NaT)
+                    # lti_df = lti_df.apply(lambda x: x.dropna().max(), axis=1)
 
-                    lti_df.set_index(entity.index, inplace=True)
-                    lti_df = lti_df.reindex(entity.last_time_index.index)
-                    lti_df['last_time_old'] = entity.last_time_index
-                    lti_df = lti_df.apply(lambda x: x.dropna().max(), axis=1)
                     entity.last_time_index = lti_df
                     entity.last_time_index.name = 'last_time'
 
