@@ -2,6 +2,7 @@ from datetime import datetime, timedelta
 
 import numpy as np
 import pandas as pd
+from dask import dataframe as dd
 from scipy import stats
 
 from featuretools.primitives.base.aggregation_primitive_base import (
@@ -32,10 +33,14 @@ class Count(AggregationPrimitive):
     return_type = Numeric
     stack_on_self = False
     default_value = 0
+    dask_compatible = True
 
     def get_function(self):
         return pd.Series.count
 
+    def get_dask_aggregation(self):
+        return 'count'
+
     def generate_name(self, base_feature_names, relationship_path_name,
                       parent_entity_id, where_str, use_prev_str):
         return u"COUNT(%s%s%s)" % (relationship_path_name,
@@ -56,10 +61,14 @@ class Sum(AggregationPrimitive):
     stack_on_self = False
     stack_on_exclude = [Count]
     default_value = 0
+    dask_compatible = True
 
     def get_function(self):
         return np.sum
 
+    def get_dask_aggregation(self):
+        return 'sum'
+
 
 class Mean(AggregationPrimitive):
     """Computes the average for a list of values.
@@ -82,6 +91,7 @@ class Mean(AggregationPrimitive):
     name = "mean"
     input_types = [Numeric]
     return_type = Numeric
+    dask_compatible = True
 
     def __init__(self, skipna=True):
         self.skipna = skipna
@@ -96,6 +106,9 @@ class Mean(AggregationPrimitive):
 
         return mean
 
+    def get_dask_aggregation(self):
+        return 'mean'
+
 
 class Mode(AggregationPrimitive):
     """Determines the most commonly repeated value.
@@ -133,10 +146,14 @@ class Min(AggregationPrimitive):
     input_types = [Numeric]
     return_type = Numeric
     stack_on_self = False
+    dask_compatible = True
 
     def get_function(self):
         return np.min
 
+    def get_dask_aggregation(self):
+        return 'min'
+
 
 class Max(AggregationPrimitive):
     """Calculates the highest value, ignoring `NaN` values.
@@ -150,10 +167,14 @@ class Max(AggregationPrimitive):
     input_types = [Numeric]
     return_type = Numeric
     stack_on_self = False
+    dask_compatible = True
 
     def get_function(self):
         return np.max
 
+    def get_dask_aggregation(self):
+        return 'max'
+
 
 class NumUnique(AggregationPrimitive):
     """Determines the number of distinct values, ignoring `NaN` values.
@@ -172,10 +193,31 @@ class NumUnique(AggregationPrimitive):
     input_types = [Discrete]
     return_type = Numeric
     stack_on_self = False
+    dask_compatible = True
 
     def get_function(self):
         return pd.Series.nunique
 
+    def get_dask_aggregation(self):
+        def chunk(s):
+            def inner_chunk(x):
+                x = x[:].dropna()
+                return set(x.unique())
+
+            return s.agg(inner_chunk)
+
+        def agg(s):
+            def inner_agg(x):
+                x = x[:].dropna()
+                return(set().union(*x.values))
+
+            return s.agg(inner_agg)
+
+        def finalize(s):
+            return s.apply(lambda x: len(x))
+
+        return dd.Aggregation(self.name, chunk=chunk, agg=agg, finalize=finalize)
+
 
 class NumTrue(AggregationPrimitive):
     """Counts the number of `True` values.
@@ -195,10 +237,23 @@ class NumTrue(AggregationPrimitive):
     default_value = 0
     stack_on = []
     stack_on_exclude = []
+    dask_compatible = True
 
     def get_function(self):
         return np.sum
 
+    def get_dask_aggregation(self):
+        def chunk(s):
+            chunk_sum = s.agg(np.sum)
+            if chunk_sum.dtype == 'bool':
+                chunk_sum = chunk_sum.astype('int64')
+            return chunk_sum
+
+        def agg(s):
+            return s.agg(np.sum)
+
+        return dd.Aggregation(self.name, chunk=chunk, agg=agg)
+
 
 class PercentTrue(AggregationPrimitive):
     """Determines the percent of `True` values.
@@ -220,6 +275,7 @@ class PercentTrue(AggregationPrimitive):
     stack_on = []
     stack_on_exclude = []
     default_value = 0
+    dask_compatible = True
 
     def get_function(self):
         def percent_true(s):
@@ -227,6 +283,27 @@ class PercentTrue(AggregationPrimitive):
 
         return percent_true
 
+    def get_dask_aggregation(self):
+        def chunk(s):
+            def format_chunk(x):
+                return x[:].fillna(0)
+
+            chunk_sum = s.agg(lambda x: format_chunk(x).sum())
+            chunk_len = s.agg(lambda x: len(format_chunk(x)))
+            if chunk_sum.dtype == 'bool':
+                chunk_sum = chunk_sum.astype('int64')
+            if chunk_len.dtype == 'bool':
+                chunk_len = chunk_len.astype('int64')
+            return (chunk_sum, chunk_len)
+
+        def agg(val, length):
+            return (val.sum(), length.sum())
+
+        def finalize(total, length):
+            return total / length
+
+        return dd.Aggregation(self.name, chunk=chunk, agg=agg, finalize=finalize)
+
 
 class NMostCommon(AggregationPrimitive):
     """Determines the `n` most common elements.
@@ -385,10 +462,14 @@ class Std(AggregationPrimitive):
     input_types = [Numeric]
     return_type = Numeric
     stack_on_self = False
+    dask_compatible = True
 
     def get_function(self):
         return np.std
 
+    def get_dask_aggregation(self):
+        return 'std'
+
 
 class First(AggregationPrimitive):
     """Determines the first value in a list.
@@ -446,10 +527,20 @@ class Any(AggregationPrimitive):
     input_types = [Boolean]
     return_type = Boolean
     stack_on_self = False
+    dask_compatible = True
 
     def get_function(self):
         return np.any
 
+    def get_dask_aggregation(self):
+        def chunk(s):
+            return s.agg(np.any)
+
+        def agg(s):
+            return s.agg(np.any)
+
+        return dd.Aggregation(self.name, chunk=chunk, agg=agg)
+
 
 class All(AggregationPrimitive):
     """Calculates if all values are 'True' in a list.
@@ -467,10 +558,20 @@ class All(AggregationPrimitive):
     input_types = [Boolean]
     return_type = Boolean
     stack_on_self = False
+    dask_compatible = True
 
     def get_function(self):
         return np.all
 
+    def get_dask_aggregation(self):
+        def chunk(s):
+            return s.agg(np.all)
+
+        def agg(s):
+            return s.agg(np.all)
+
+        return dd.Aggregation(self.name, chunk=chunk, agg=agg)
+
 
 class TimeSinceLast(AggregationPrimitive):
     """Calculates the time elapsed since the last datetime (default in seconds).
