@@ -1,3 +1,4 @@
+import dask.dataframe as dd
 import numpy as np
 import pandas as pd
 
@@ -5,6 +6,7 @@ from featuretools.primitives.base.transform_primitive_base import (
     TransformPrimitive
 )
 from featuretools.utils import convert_time_units
+from featuretools.utils.entity_utils import replace_latlong_nan
 from featuretools.variable_types import (
     Boolean,
     DateOfBirth,
@@ -29,9 +31,15 @@ class IsNull(TransformPrimitive):
     name = "is_null"
     input_types = [Variable]
     return_type = Boolean
+    dask_compatible = True
 
     def get_function(self):
-        return lambda array: pd.isnull(pd.Series(array))
+        def isnull(array):
+            if isinstance(array, dd.Series):
+                return dd.Series.isnull(array)
+            else:
+                return pd.isnull(pd.Series(array))
+        return isnull
 
 
 class Absolute(TransformPrimitive):
@@ -45,6 +53,7 @@ class Absolute(TransformPrimitive):
     name = "absolute"
     input_types = [Numeric]
     return_type = Numeric
+    dask_compatible = True
 
     def get_function(self):
         return np.absolute
@@ -102,9 +111,12 @@ class Day(TransformPrimitive):
     name = "day"
     input_types = [Datetime]
     return_type = Ordinal
+    dask_compatible = True
 
     def get_function(self):
         def day(vals):
+            if isinstance(vals, dd.Series):
+                return vals.dt.day
             return pd.DatetimeIndex(vals).day.values
         return day
 
@@ -124,9 +136,12 @@ class Hour(TransformPrimitive):
     name = "hour"
     input_types = [Datetime]
     return_type = Ordinal
+    dask_compatible = True
 
     def get_function(self):
         def hour(vals):
+            if isinstance(vals, dd.Series):
+                return vals.dt.hour
             return pd.DatetimeIndex(vals).hour.values
         return hour
 
@@ -146,9 +161,12 @@ class Second(TransformPrimitive):
     name = "second"
     input_types = [Datetime]
     return_type = Numeric
+    dask_compatible = True
 
     def get_function(self):
         def second(vals):
+            if isinstance(vals, dd.Series):
+                return vals.dt.second
             return pd.DatetimeIndex(vals).second.values
         return second
 
@@ -168,9 +186,12 @@ class Minute(TransformPrimitive):
     name = "minute"
     input_types = [Datetime]
     return_type = Numeric
+    dask_compatible = True
 
     def get_function(self):
         def minute(vals):
+            if isinstance(vals, dd.Series):
+                return vals.dt.minute
             return pd.DatetimeIndex(vals).minute.values
         return minute
 
@@ -195,9 +216,12 @@ class Week(TransformPrimitive):
     name = "week"
     input_types = [Datetime]
     return_type = Ordinal
+    dask_compatible = True
 
     def get_function(self):
         def week(vals):
+            if isinstance(vals, dd.Series):
+                return vals.dt.week
             return pd.DatetimeIndex(vals).week.values
         return week
 
@@ -217,9 +241,12 @@ class Month(TransformPrimitive):
     name = "month"
     input_types = [Datetime]
     return_type = Ordinal
+    dask_compatible = True
 
     def get_function(self):
         def month(vals):
+            if isinstance(vals, dd.Series):
+                return vals.dt.month
             return pd.DatetimeIndex(vals).month.values
         return month
 
@@ -239,9 +266,12 @@ class Year(TransformPrimitive):
     name = "year"
     input_types = [Datetime]
     return_type = Ordinal
+    dask_compatible = True
 
     def get_function(self):
         def year(vals):
+            if isinstance(vals, dd.Series):
+                return vals.dt.year
             return pd.DatetimeIndex(vals).year.values
         return year
 
@@ -261,9 +291,12 @@ class IsWeekend(TransformPrimitive):
     name = "is_weekend"
     input_types = [Datetime]
     return_type = Boolean
+    dask_compatible = True
 
     def get_function(self):
         def is_weekend(vals):
+            if isinstance(vals, dd.Series):
+                return vals.dt.weekday > 4
             return pd.DatetimeIndex(vals).weekday.values > 4
         return is_weekend
 
@@ -287,9 +320,12 @@ class Weekday(TransformPrimitive):
     name = "weekday"
     input_types = [Datetime]
     return_type = Ordinal
+    dask_compatible = True
 
     def get_function(self):
         def weekday(vals):
+            if isinstance(vals, dd.Series):
+                return vals.dt.weekday
             return pd.DatetimeIndex(vals).weekday.values
         return weekday
 
@@ -307,9 +343,12 @@ class NumCharacters(TransformPrimitive):
     name = 'num_characters'
     input_types = [Text]
     return_type = Numeric
+    dask_compatible = True
 
     def get_function(self):
-        return lambda array: pd.Series(array).fillna('').str.len()
+        def character_counter(array):
+            return array.fillna('').str.len()
+        return character_counter
 
 
 class NumWords(TransformPrimitive):
@@ -326,10 +365,11 @@ class NumWords(TransformPrimitive):
     name = 'num_words'
     input_types = [Text]
     return_type = Numeric
+    dask_compatible = True
 
     def get_function(self):
         def word_counter(array):
-            return pd.Series(array).fillna('').str.count(' ') + 1
+            return array.fillna('').str.count(' ') + 1
         return word_counter
 
 
@@ -368,13 +408,16 @@ class TimeSince(TransformPrimitive):
     input_types = [[DatetimeTimeIndex], [Datetime]]
     return_type = Numeric
     uses_calc_time = True
+    dask_compatible = True
 
     def __init__(self, unit="seconds"):
         self.unit = unit.lower()
 
     def get_function(self):
         def pd_time_since(array, time):
-            return convert_time_units((time - pd.DatetimeIndex(array)).total_seconds(), self.unit)
+            if isinstance(array, list):
+                array = pd.Series(array)
+            return convert_time_units((time - array).dt.total_seconds(), self.unit)
         return pd_time_since
 
 
@@ -390,13 +433,17 @@ class IsIn(TransformPrimitive):
     name = "isin"
     input_types = [Variable]
     return_type = Boolean
+    dask_compatible = True
 
     def __init__(self, list_of_outputs=None):
         self.list_of_outputs = list_of_outputs
 
     def get_function(self):
         def pd_is_in(array):
-            return pd.Series(array).isin(self.list_of_outputs or [])
+            if isinstance(array, dd.Series):
+                return array.isin(self.list_of_outputs or [])
+            else:
+                return pd.Series(array).isin(self.list_of_outputs or [])
         return pd_is_in
 
     def generate_name(self, base_feature_names):
@@ -442,6 +489,7 @@ class Negate(TransformPrimitive):
     name = "negate"
     input_types = [Numeric]
     return_type = Numeric
+    dask_compatible = True
 
     def get_function(self):
         def negate(vals):
@@ -463,6 +511,7 @@ class Not(TransformPrimitive):
     name = "not"
     input_types = [Boolean]
     return_type = Boolean
+    dask_compatible = True
 
     def generate_name(self, base_feature_names):
         return u"NOT({})".format(base_feature_names[0])
@@ -509,7 +558,11 @@ class Latitude(TransformPrimitive):
     return_type = Numeric
 
     def get_function(self):
-        return lambda array: pd.Series([x[0] for x in array])
+        def latitude(latlong):
+            if latlong.hasnans:
+                latlong = replace_latlong_nan(latlong)
+            return pd.Series(x[0] for x in latlong)
+        return latitude
 
 
 class Longitude(TransformPrimitive):
@@ -528,7 +581,11 @@ class Longitude(TransformPrimitive):
     return_type = Numeric
 
     def get_function(self):
-        return lambda array: pd.Series([x[1] for x in array])
+        def longitude(latlong):
+            if latlong.hasnans:
+                latlong = replace_latlong_nan(latlong)
+            return pd.Series(x[1] for x in latlong)
+        return longitude
 
 
 class Haversine(TransformPrimitive):
@@ -568,6 +625,10 @@ class Haversine(TransformPrimitive):
 
     def get_function(self):
         def haversine(latlong1, latlong2):
+            if latlong1.hasnans:
+                latlong1 = replace_latlong_nan(latlong1)
+            if latlong2.hasnans:
+                latlong2 = replace_latlong_nan(latlong2)
             lat_1s = np.array([x[0] for x in latlong1])
             lon_1s = np.array([x[1] for x in latlong1])
             lat_2s = np.array([x[0] for x in latlong2])
