@@ -3,6 +3,7 @@ from datetime import datetime
 import numpy as np
 import pandas as pd
 import pytest
+from dask import dataframe as dd
 
 import featuretools as ft
 from featuretools import variable_types
@@ -59,7 +60,7 @@ def test_eq(es):
 
     assert es['log'].__eq__(es['log'], deep=True)
     assert es['log'].__eq__(other_es['log'], deep=True)
-    assert (es['log'].df['latlong'] == latlong).all()
+    assert all(es['log'].df['latlong'].eq(latlong))
 
     other_es['log'].add_interesting_values()
     assert not es['log'].__eq__(other_es['log'], deep=True)
@@ -81,7 +82,7 @@ def test_eq(es):
 
 def test_update_data(es):
     df = es['customers'].df.copy()
-    df['new'] = [1, 2, 3]
+    df['new'] = pd.Series([1, 2, 3])
 
     error_text = 'Updated dataframe is missing new cohort column'
     with pytest.raises(ValueError, match=error_text):
@@ -92,20 +93,42 @@ def test_update_data(es):
         es['customers'].update_data(df)
 
     # test already_sorted on entity without time index
-    df = es["sessions"].df.copy(deep=True)
-    df["id"].iloc[1:3] = [2, 1]
-    es["sessions"].update_data(df.copy(deep=True))
-    assert es["sessions"].df["id"].iloc[1] == 2  # no sorting since time index not defined
-    es["sessions"].update_data(df.copy(deep=True), already_sorted=True)
-    assert es["sessions"].df["id"].iloc[1] == 2
+    df = es["sessions"].df.copy()
+    if isinstance(df, dd.DataFrame):
+        updated_id = df['id'].compute()
+    else:
+        updated_id = df['id']
+    updated_id.iloc[1:3] = [2, 1]
+    df["id"] = updated_id
+    es["sessions"].update_data(df.copy())
+    sessions_df = es['sessions'].df
+    if isinstance(sessions_df, dd.DataFrame):
+        sessions_df = sessions_df.compute()
+    assert sessions_df["id"].iloc[1] == 2  # no sorting since time index not defined
+    es["sessions"].update_data(df.copy(), already_sorted=True)
+    sessions_df = es['sessions'].df
+    if isinstance(sessions_df, dd.DataFrame):
+        sessions_df = sessions_df.compute()
+    assert sessions_df["id"].iloc[1] == 2
 
     # test already_sorted on entity with time index
-    df = es["customers"].df.copy(deep=True)
-    df["signup_date"].iloc[0] = datetime(2011, 4, 11)
-    es["customers"].update_data(df.copy(deep=True))
-    assert es["customers"].df["id"].iloc[0] == 0
-    es["customers"].update_data(df.copy(deep=True), already_sorted=True)
-    assert es["customers"].df["id"].iloc[0] == 2
+    df = es["customers"].df.copy()
+    if isinstance(df, dd.DataFrame):
+        updated_signup = df['signup_date'].compute()
+    else:
+        updated_signup = df['signup_date']
+    updated_signup.iloc[0] = datetime(2011, 4, 11)
+    df['signup_date'] = updated_signup
+    es["customers"].update_data(df.copy(), already_sorted=True)
+    customers_df = es['customers'].df
+    if isinstance(customers_df, dd.DataFrame):
+        customers_df = customers_df.compute()
+    assert customers_df["id"].iloc[0] == 2
+
+    # only pandas allows for sorting:
+    if isinstance(df, pd.DataFrame):
+        es["customers"].update_data(df.copy())
+        assert es['customers'].df["id"].iloc[0] == 0
 
 
 def test_query_by_values_returns_rows_in_given_order():
@@ -118,7 +141,7 @@ def test_query_by_values_returns_rows_in_given_order():
     es = ft.EntitySet()
     es = es.entity_from_dataframe(entity_id="test", dataframe=data, index="id",
                                   time_index="time", variable_types={
-                                      "value": ft.variable_types.Categorical
+                                            "value": ft.variable_types.Categorical
                                   })
     query = es['test'].query_by_values(['b', 'a'], variable_id='value')
     assert np.array_equal(query['id'], [1, 3, 4, 5])
@@ -129,6 +152,8 @@ def test_query_by_values_secondary_time_index(es):
     all_instances = [0, 1, 2]
     result = es['customers'].query_by_values(all_instances, time_last=end)
 
+    if isinstance(result, dd.DataFrame):
+        result = result.compute().set_index('id')
     for col in ["cancel_date", "cancel_reason"]:
         nulls = result.loc[all_instances][col].isnull() == [False, True, True]
         assert nulls.all(), "Some instance has data it shouldn't for column %s" % col
@@ -143,7 +168,7 @@ def test_delete_variables(es):
 
     for var in to_delete:
         assert var not in variable_names
-        assert var not in entity.df
+        assert var not in entity.df.columns
 
 
 def test_variable_types_unmodified():
@@ -223,3 +248,13 @@ def test_passing_strings_to_variable_types_dfs():
     features = ft.dfs(entities, relationships, target_entity="teams", features_only=True)
     name_class = features[0].entity['name'].__class__
     assert name_class == variable_types['text']
+
+
+def test_replace_latlong_nan_during_entity_creation(pd_es):
+    nan_es = ft.EntitySet("latlong_nan")
+    df = pd_es['log'].df.copy()
+    df['latlong'][0] = np.nan
+
+    with pytest.warns(UserWarning, match="LatLong columns should contain only tuples. All single 'NaN' values in column 'latlong' have been replaced with '\\(NaN, NaN\\)'."):
+        entity = ft.Entity(id="nan_latlong_entity", df=df, entityset=nan_es, variable_types=pd_es['log'].variable_types)
+    assert entity.df['latlong'][0] == (np.nan, np.nan)
